import { Writable } from 'node:stream';

/**
 * TerminalEmulator provides a terminal-like interface for CLI applications, allowing users
 * to interact with a simulated terminal environment. It supports processing user input through
 * a customizable input handler function, enabling the execution of commands and the return of
 * various output types (string, object, or Error). The class also allows for the subscription
 * of external writable streams for standard output (stdout) and standard error (stderr),
 * facilitating integration with other output handling systems. Users can set the current user
 * and directory for the terminal prompt, clear the terminal screen, and gracefully handle
 * the Ctrl+C interrupt signal.
 *
 * Intended usage:
 * 1. Create an instance of TerminalEmulator with a user-defined input handler.
 * 2. Use the `start` method to begin the terminal interaction.
 * 3. Optionally, subscribe writable streams to capture output.
 * 4. Customize user and directory settings as needed.
 * 5. Handle user input and commands within the provided input handler.
 *
 * Example:
 * const inputHandler = (input: string) => {
 *   if (input.startsWith('error')) {
 *     return new Error('This is an error message.'); // Return an error object
 *   }
 *   if (input === 'object') {
 *     return {
 *       key1: 'value1',
 *       key2: 'value2',
 *       key3: 'This is a long value that will exceed the 80 character limit and should be wrapped accordingly.',
 *     }; // Return a plain object
 *   }
 *   return `You typed: ${input}`; // Return a string for stdout
 * };
 *
 * const terminal = new TerminalEmulator('user', '/home/user', inputHandler);
 * terminal.start(); // Starts the terminal emulator
 */
/**
 * Type for the input handler function.
 * It receives a string input and returns either a string, an object, or an Error.
 */
type InputHandler$1 = (input: string) => string | object | Error | undefined;
declare class TerminalEmulator {
    user: string;
    currentDirectory: string;
    private inputHandler?;
    private history;
    private rl;
    private stdoutStream;
    private stderrStream;
    /**
     * Creates an instance of TerminalEmulator.
     * @param user - The user for the terminal prompt. Defaults to 'guest'.
     * @param currentDirectory - The current directory for the terminal prompt. Defaults to '/'.
     * @param inputHandler - A function to handle user input. It receives the input and returns output.
     */
    constructor(user?: string, currentDirectory?: string, inputHandler?: InputHandler$1);
    /**
     * Starts the terminal emulator, displaying the prompt for user input.
     */
    start(): void;
    /**
     * Clears the terminal screen and resets the input history.
     */
    clearTerminal(): void;
    /**
     * Exits the terminal emulator.
     */
    exit(): void;
    /**
     * Sets the user for the terminal prompt.
     * @param user - The new user name for the prompt.
     */
    setUser(user: string): void;
    /**
     * Sets the current directory for the terminal prompt.
     * @param directory - The new current directory for the prompt.
     */
    setCurrentDirectory(directory: string): void;
    /**
     * Subscribes a writable stream to receive stdout messages in the terminal.
     * @param stream - The stream to subscribe to.
     */
    subscribeStdout(stream: Writable): void;
    /**
     * Subscribes a writable stream to receive stderr messages in the terminal.
     * @param stream - The stream to subscribe to.
     */
    subscribeStderr(stream: Writable): void;
    /**
     * Clears the terminal screen based on the operating system.
     * @private
     */
    private clearScreen;
    /**
     * Handles user input and processes commands.
     * @param input - The input from the user.
     * @private
     */
    private handleInput;
    /**
     * Checks if the given output is an error object.
     * @param output - The output to check.
     * @returns True if it's an error object, false otherwise.
     * @private
     */
    private isError;
    /**
     * Checks if the given output is a plain object.
     * @param output - The output to check.
     * @returns True if it's a plain object, false otherwise.
     * @private
     */
    private isPlainObject;
    /**
     * Prints the standard output to the terminal or subscribed stream.
     * @param output - The standard output to print.
     * @private
     */
    private printOutput;
    /**
     * Prints the error message to the terminal or subscribed stream.
     * @param error - The error object to print.
     * @private
     */
    private printError;
    /**
     * Pretty prints a plain object into the terminal, wrapping lines at 80 characters.
     * @param obj - The object to print.
     * @private
     */
    private printObject;
    /**
     * Wraps text to a specified column width.
     * @param text - The text to wrap.
     * @param width - The column width to wrap the text at.
     * @returns An array of wrapped lines.
     * @private
     */
    private wrapText;
    /**
     * Updates the prompt after printing output without clearing the input history.
     * @private
     */
    private updatePrompt;
    /**
     * Handles the Ctrl+C signal to pause the terminal interaction.
     * @private
     */
    private handleSigint;
    /**
     * Writes a message to standard output or the subscribed stream.
     * @param message - The message to write to stdout.
     * @private
     */
    private writeToStdout;
    /**
     * Writes a message to standard error or the subscribed stream.
     * @param message - The message to write to stderr.
     * @private
     */
    private writeToStderr;
}

/**
 * Type for the input handler function.
 * It receives a string input and returns either a string, an object, or an Error.
 */
type InputHandler = (input: string) => string | object | Error | undefined;
/**
 * Interface for the TerminalEmulator class.
 * Provides methods for interacting with the terminal emulator.
 */
interface ITerminalEmulator {
    user: string;
    currentDirectory: string;
    /**
     * Starts the terminal emulator, displaying the prompt for user input.
     */
    start(): void;
    /**
     * Clears the terminal screen and resets the input history.
     */
    clearTerminal(): void;
    /**
     * Exits the terminal emulator.
     */
    exit(): void;
    /**
     * Sets the user for the terminal prompt.
     * @param user - The new user name for the prompt.
     */
    setUser(user: string): void;
    /**
     * Sets the current directory for the terminal prompt.
     * @param directory - The new current directory for the prompt.
     */
    setCurrentDirectory(directory: string): void;
    /**
     * Subscribes a writable stream to receive stdout messages in the terminal.
     * @param stream - The stream to subscribe to.
     */
    subscribeStdout(stream: Writable): void;
    /**
     * Subscribes a writable stream to receive stderr messages in the terminal.
     * @param stream - The stream to subscribe to.
     */
    subscribeStderr(stream: Writable): void;
}

export { type ITerminalEmulator, type InputHandler, TerminalEmulator };
