// src/terminal-emulator.ts
import readline from "node:readline";
import { exec } from "node:child_process";
var TerminalEmulator = class {
  // =======================
  // Public Properties
  // =======================
  user;
  currentDirectory;
  // =======================
  // Private Properties
  // =======================
  inputHandler;
  history;
  rl;
  stdoutStream = null;
  // Writable stream for stdout
  stderrStream = null;
  // Writable stream for stderr
  // =======================
  // Public Methods
  // =======================
  /**
   * Creates an instance of TerminalEmulator.
   * @param user - The user for the terminal prompt. Defaults to 'guest'.
   * @param currentDirectory - The current directory for the terminal prompt. Defaults to '/'.
   * @param inputHandler - A function to handle user input. It receives the input and returns output.
   */
  constructor(user = "guest", currentDirectory = "/", inputHandler) {
    this.user = user;
    this.currentDirectory = currentDirectory;
    this.inputHandler = inputHandler;
    this.history = [];
    this.rl = readline.createInterface({
      input: process.stdin,
      output: process.stdout,
      prompt: `${this.user}@server:${this.currentDirectory}$ `
    });
    this.clearScreen();
    process.on("SIGINT", () => {
      this.handleSigint();
    });
  }
  /**
   * Starts the terminal emulator, displaying the prompt for user input.
   */
  start() {
    this.rl.prompt();
    this.rl.on("line", (input) => {
      this.handleInput(input);
      this.rl.prompt();
    }).on("close", () => {
      this.exit();
    });
  }
  /**
   * Clears the terminal screen and resets the input history.
   */
  clearTerminal() {
    this.clearScreen();
    this.history = [];
    this.updatePrompt();
  }
  /**
   * Exits the terminal emulator.
   */
  exit() {
    this.writeToStdout("Exiting terminal emulator...");
    this.rl.close();
  }
  /**
   * Sets the user for the terminal prompt.
   * @param user - The new user name for the prompt.
   */
  setUser(user) {
    this.user = user;
    this.updatePrompt();
  }
  /**
   * Sets the current directory for the terminal prompt.
   * @param directory - The new current directory for the prompt.
   */
  setCurrentDirectory(directory) {
    this.currentDirectory = directory;
    this.updatePrompt();
  }
  /**
   * Subscribes a writable stream to receive stdout messages in the terminal.
   * @param stream - The stream to subscribe to.
   */
  subscribeStdout(stream) {
    this.stdoutStream = stream;
  }
  /**
   * Subscribes a writable stream to receive stderr messages in the terminal.
   * @param stream - The stream to subscribe to.
   */
  subscribeStderr(stream) {
    this.stderrStream = stream;
  }
  // =======================
  // Private Methods
  // =======================
  /**
   * Clears the terminal screen based on the operating system.
   * @private
   */
  clearScreen() {
    const platform = process.platform;
    const clearCommand = platform === "win32" ? "cls" : "clear";
    exec(clearCommand, (err) => {
      if (err) {
        this.writeToStderr(`Error clearing screen: ${err}`);
      }
    });
  }
  /**
   * Handles user input and processes commands.
   * @param input - The input from the user.
   * @private
   */
  handleInput(input) {
    this.history.push(input);
    if (this.inputHandler) {
      const output = this.inputHandler(input);
      if (this.isError(output)) {
        this.printError(output);
      } else if (this.isPlainObject(output)) {
        this.printObject(output);
      } else if (typeof output === "string") {
        this.printOutput(output);
      }
    }
  }
  /**
   * Checks if the given output is an error object.
   * @param output - The output to check.
   * @returns True if it's an error object, false otherwise.
   * @private
   */
  isError(output) {
    return output instanceof Error;
  }
  /**
   * Checks if the given output is a plain object.
   * @param output - The output to check.
   * @returns True if it's a plain object, false otherwise.
   * @private
   */
  isPlainObject(output) {
    return output !== null && typeof output === "object" && !Array.isArray(output);
  }
  /**
   * Prints the standard output to the terminal or subscribed stream.
   * @param output - The standard output to print.
   * @private
   */
  printOutput(output) {
    this.writeToStdout(output);
    this.updatePrompt();
  }
  /**
   * Prints the error message to the terminal or subscribed stream.
   * @param error - The error object to print.
   * @private
   */
  printError(error) {
    this.writeToStderr(error.message);
    this.updatePrompt();
  }
  /**
   * Pretty prints a plain object into the terminal, wrapping lines at 80 characters.
   * @param obj - The object to print.
   * @private
   */
  printObject(obj) {
    const objectString = JSON.stringify(obj, null, 2);
    const lines = this.wrapText(objectString, 80);
    for (const line of lines) {
      this.writeToStdout(line);
    }
    this.updatePrompt();
  }
  /**
   * Wraps text to a specified column width.
   * @param text - The text to wrap.
   * @param width - The column width to wrap the text at.
   * @returns An array of wrapped lines.
   * @private
   */
  wrapText(text, width) {
    const regex = new RegExp(`(.{1,${width}})(\\s|$)`, "g");
    const matches = text.match(regex);
    return matches || [text];
  }
  /**
   * Updates the prompt after printing output without clearing the input history.
   * @private
   */
  updatePrompt() {
    this.rl.setPrompt(`${this.user}@server:${this.currentDirectory}$ `);
    this.rl.prompt();
  }
  /**
   * Handles the Ctrl+C signal to pause the terminal interaction.
   * @private
   */
  handleSigint() {
    this.rl.write("\n");
    this.writeToStdout("(Press Ctrl+D to exit)");
    this.rl.setPrompt(`${this.user}@server:${this.currentDirectory} (Press Ctrl+D to exit)$ `);
    this.rl.prompt();
  }
  /**
   * Writes a message to standard output or the subscribed stream.
   * @param message - The message to write to stdout.
   * @private
   */
  writeToStdout(message) {
    const outputMessage = `${message}
`;
    if (this.stdoutStream) {
      this.stdoutStream.write(outputMessage);
    } else {
      process.stdout.write(outputMessage);
    }
  }
  /**
   * Writes a message to standard error or the subscribed stream.
   * @param message - The message to write to stderr.
   * @private
   */
  writeToStderr(message) {
    const errorMessage = `${message}
`;
    if (this.stderrStream) {
      this.stderrStream.write(errorMessage);
    } else {
      process.stderr.write(errorMessage);
    }
  }
};
export {
  TerminalEmulator
};
